@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Path.IO
--- Copyright   :  © 2016 Mark Karpov
+-- Copyright   :  © 2016–2017 Mark Karpov
 -- License     :  BSD 3 clause
 --
 -- Maintainer  :  Mark Karpov <markkarpov@openmailbox.org>
@@ -29,7 +29,7 @@ module Path.IO
   , copyDirRecur
   , copyDirRecur'
     -- ** Walking directory trees
-  , WalkAction(..)
+  , WalkAction (..)
   , walkDir
   , walkDirAccum
     -- ** Current working directory
@@ -41,6 +41,10 @@ module Path.IO
   , getAppUserDataDir
   , getUserDocsDir
   , getTempDir
+#if MIN_VERSION_directory(1,2,3)
+  , XdgDirectory (..)
+  , getXdgDir
+#endif
     -- * Path transformation
   , AbsPath
   , RelPath
@@ -99,7 +103,7 @@ import Control.Monad.Catch
 import Control.Monad.IO.Class (MonadIO (..))
 import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.Maybe (MaybeT (..), runMaybeT)
-import Control.Monad.Trans.Writer.Lazy (runWriterT, tell)
+import Control.Monad.Trans.Writer.Lazy (execWriterT, tell)
 import Data.Either (lefts, rights)
 import Data.List ((\\))
 import Data.Time (UTCTime)
@@ -107,6 +111,9 @@ import Path
 import System.IO (Handle)
 import System.IO.Error (isDoesNotExistError)
 import System.PosixCompat.Files (deviceID, fileID, getFileStatus)
+#if MIN_VERSION_directory(1,2,3)
+import System.Directory (XdgDirectory)
+#endif
 import qualified Data.Set         as S
 import qualified System.Directory as D
 import qualified System.FilePath  as F
@@ -430,9 +437,10 @@ data WalkAction
   | WalkExclude [Path Abs Dir]  -- ^ List of sub-directories to exclude from
                                 -- descending
 
--- | Traverse a directory tree, calling a handler function at each directory
--- node traversed. The absolute paths of the parent directory, sub-directories
--- and the files in the directory are provided as arguments to the handler.
+-- | Traverse a directory tree using depth first pre-order traversal, calling a
+-- handler function at each directory node traversed. The absolute paths of the
+-- parent directory, sub-directories and the files in the directory are
+-- provided as arguments to the handler.
 --
 -- Detects and silently avoids any traversal loops in the directory tree.
 --
@@ -496,8 +504,7 @@ walkDirAccum
      -- ^ Directory where traversal begins
   -> m o
      -- ^ Accumulation of outputs generated by the output writer invocations
-walkDirAccum dHandler writer topdir =
-  liftM snd . runWriterT $ walkDir handler topdir
+walkDirAccum dHandler writer topdir = execWriterT $ walkDir handler topdir
   where
     handler dir subdirs files = do
       res <- lift $ writer dir subdirs files
@@ -700,6 +707,34 @@ getTempDir :: (MonadIO m, MonadThrow m) => m (Path Abs Dir)
 getTempDir = liftIO D.getTemporaryDirectory >>= resolveDir'
 {-# INLINE getTempDir #-}
 
+#if MIN_VERSION_directory(1,2,3)
+-- | Obtain the paths to special directories for storing user-specific
+-- application data, configuration, and cache files, conforming to the
+-- <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html XDG Base Directory Specification>.
+-- Compared with 'getAppUserDataDir', this function provides a more
+-- fine-grained hierarchy as well as greater flexibility for the user.
+--
+-- It also works on Windows, although in that case 'XdgData' and 'XdgConfig'
+-- will map to the same directory.
+--
+-- Note: The directory may not actually exist, in which case you would need
+-- to create it with file mode @700@ (i.e. only accessible by the owner).
+--
+-- Note also: this is a piece of conditional API, only available if
+-- @directory-1.2.3.0@ or later is used.
+--
+-- @since 1.2.1
+
+getXdgDir :: (MonadIO m, MonadThrow m)
+  => XdgDirectory      -- ^ Which special directory
+  -> Maybe (Path Rel Dir)
+     -- ^ A relative path that is appended to the path; if 'Nothing', the
+     -- base path is returned
+  -> m (Path Abs Dir)
+getXdgDir xdgDir suffix = liftIO (D.getXdgDirectory xdgDir $ maybe "" toFilePath suffix) >>= parseAbsDir
+{-# INLINE getXdgDir #-}
+#endif
+
 ----------------------------------------------------------------------------
 -- Path transformation
 
@@ -1243,6 +1278,9 @@ copyPermissions = liftD2 D.copyPermissions
 -- Caveat for POSIX systems: This function returns a timestamp with
 -- sub-second resolution only if this package is compiled against
 -- @unix-2.6.0.0@ or later and the underlying filesystem supports them.
+--
+-- Note: this is a piece of conditional API, only available if
+-- @directory-1.2.3.0@ or later is used.
 
 getAccessTime :: MonadIO m => Path b t -> m UTCTime
 getAccessTime = liftD D.getAccessTime
@@ -1269,6 +1307,9 @@ getAccessTime = liftD D.getAccessTime
 --   function would not be able to set timestamps with sub-second
 --   resolution. In this case, there would also be loss of precision in the
 --   modification time.
+--
+-- Note: this is a piece of conditional API, only available if
+-- @directory-1.2.3.0@ or later is used.
 
 setAccessTime :: MonadIO m => Path b t -> UTCTime -> m ()
 setAccessTime = liftD2' D.setAccessTime
@@ -1295,9 +1336,12 @@ setAccessTime = liftD2' D.setAccessTime
 --   function would not be able to set timestamps with sub-second
 --   resolution. In this case, there would also be loss of precision in the
 --   access time.
+--
+-- Note: this is a piece of conditional API, only available if
+-- @directory-1.2.3.0@ or later is used.
 
 setModificationTime :: MonadIO m => Path b t -> UTCTime -> m ()
-setModificationTime = liftD2' D.setAccessTime
+setModificationTime = liftD2' D.setModificationTime
 {-# INLINE setModificationTime #-}
 #endif
 
